#include <System/Cpu/Idt.h>

idtGate mainIdt[IDT_ENTRIES];
idtRegister mainIdtReg;

void setIdt() {
    mainIdtReg.base = (uint64_t) &mainIdt;
    mainIdtReg.limit = (IDT_ENTRIES * sizeof(idtGate)) - 1;

    __asm__ volatile("lidt (%0)" : : "r" (&mainIdtReg));
}

void setIdtGate(uint8_t gateNumber, uint64_t handlerAddress) {
    uint16_t low16 = (uint16_t) (handlerAddress & 0xFFFF);
    uint16_t middle16 = (uint16_t) ((handlerAddress >> 16) & 0xFFFF);
    uint32_t high32 = (uint32_t) ((handlerAddress >> 32) & 0xFFFFFFFF);

    idtGate gate = {
        .baseLow = low16,
        .codeSegmentSelector = KERNEL_CS,
        .zero = 0,
        .attributes = INT_ATTR,
        .baseMiddle = middle16,
        .baseHigh = high32,
        .reserved = 0
    };

    mainIdt[gateNumber] = gate;
}